#include <stdio.h>
#include <math.h>

/***************************************************************************/
/* ran1, gasdev, qcksrt */
/***************************************************************************/

/***************************************************************************/
/* Routines obtained from Numerical Recipes in C, second edition           */
/***************************************************************************/

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran1(long *idum)
{
  int j;
  long k;
  static long iy=0;
  static long iv[NTAB];
  double temp;
  
  if (*idum <= 0 || !iy) {
    if (-(*idum) < 1) *idum=1;
    else *idum = -(*idum);
    for (j=NTAB+7;j>=0;j--) {
      k=(*idum)/IQ;
      *idum=IA*(*idum-k*IQ)-IR*k;
      if (*idum < 0) *idum += IM;
      if (j < NTAB) iv[j] = *idum;
    }
    iy=iv[0];
  }
  k=(*idum)/IQ;
  *idum=IA*(*idum-k*IQ)-IR*k;
  if (*idum < 0) *idum += IM;
  j=iy/NDIV;
  iy=iv[j];
  iv[j] = *idum;
  if ((temp=AM*iy) > RNMX) return RNMX;
  else return temp;
}
#undef IA
#undef IM
#undef AM
#undef IQ
#undef IR
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX

/***************************************************************************/

double gasdev(long *idum)
{
  double ran1(long *idum);
  static int iset=0;
  static double gset;
  double fac,rsq,v1,v2;

  if  (iset == 0) {
    do {
      v1=2.0*ran1(idum)-1.0;
      v2=2.0*ran1(idum)-1.0;
      rsq=v1*v1+v2*v2;
    } while (rsq >= 1.0 || rsq == 0.0);
    fac=sqrt(-2.0*log(rsq)/rsq);
    gset=v1*fac;
    iset=1;
    return v2*fac;
  } else {
    iset=0;
    return gset;
  }
}

/***************************************************************************/

#define M 7
#define NSTACK 50
#define FM 7875
#define FA 211
#define FC 1663

void qcksrt(n,arr)
int n;
double arr[];
{
  int l=1,jstack=0,j,ir,iq,i;
  int istack[NSTACK+1];
  long int fx=0L;
  double a;
/*  void nrerror(); */

  ir=n;
  for (;;) {
    if (ir-l < M) {
      for (j=l+1;j<=ir;j++) {
	a=arr[j];
	for (i=j-1;arr[i]>a && i>0;i--) arr[i+1]=arr[i];
	arr[i+1]=a;
      }
      if (jstack == 0) return;
      ir=istack[jstack--];
      l=istack[jstack--];
    } else {
      i=l;
      j=ir;
      fx=(fx*FA+FC) % FM;
      iq=l+((ir-l+1)*fx)/FM;
      a=arr[iq];
      arr[iq]=arr[l];
      for (;;) {
	while (j > 0 && a < arr[j]) j--;
	if (j <= i) {
	  arr[i]=a;
	  break;
	}
	arr[i++]=arr[j];
	while (a > arr[i] && i <= n) i++;
	if (j <= i) {
	  arr[(i=j)]=a;
	  break;
	}
	arr[j--]=arr[i];
      }
      if (ir-i >= i-l) {
	istack[++jstack]=i+1;
	istack[++jstack]=ir;
	ir=i-1;
      } else {
	istack[++jstack]=l;
	istack[++jstack]=i-1;
	l=i+1;
      }
      if (jstack > NSTACK) printf("NSTACK too small in QCKSRT");
    }
  }
}

#undef M
#undef NSTACK
#undef FM
#undef FA
#undef FC
